@@ -31,8 +31,6 @@
 
 #include "log.h"
 
-#include "config.h"
-
 log_define("zim.cluster")
 
 #define log_debug1(e)
@@ -75,23 +73,23 @@ getClusterReader(const Reader& zimReader, offset_t offset, Cluster::Compression*
 
 } // unnamed namespace
 
-  std::shared_ptr<Cluster> Cluster::read(const Reader& zimReader, offset_t clusterOffset)
+  std::shared_ptr<Cluster> Cluster::read(const Reader& zimReader, offset_t clusterOffset, size_t maxBlobCount)
   {
     Compression comp;
     bool extended;
     auto reader = getClusterReader(zimReader, clusterOffset, &comp, &extended);
-    return std::make_shared<Cluster>(std::move(reader), comp, extended);
+    return std::make_shared<Cluster>(std::move(reader), comp, extended, maxBlobCount);
   }
 
-  Cluster::Cluster(std::unique_ptr<IStreamReader> reader_, Compression comp, bool isExtended)
+  Cluster::Cluster(std::unique_ptr<IStreamReader> reader_, Compression comp, bool isExtended, size_t maxBlobCount)
     : compression(comp),
       isExtended(isExtended),
       m_reader(std::move(reader_))
   {
     if (isExtended) {
-      read_header<uint64_t>();
+      read_header<uint64_t>(maxBlobCount);
     } else {
-      read_header<uint32_t>();
+      read_header<uint32_t>(maxBlobCount);
     }
   }
 
@@ -99,13 +97,25 @@ getClusterReader(const Reader& zimReader, offset_t offset, Cluster::Compression*
 
   /* This return the number of char read */
   template<typename OFFSET_TYPE>
-  void Cluster::read_header()
+  void Cluster::read_header(size_t maxBlobCount)
   {
     // read first offset, which specifies, how many offsets we need to read
     OFFSET_TYPE offset = m_reader->read<OFFSET_TYPE>();
 
+    if ( offset != sizeof(OFFSET_TYPE) && offset < 2 * sizeof(OFFSET_TYPE) ) {
+        throw zim::ZimFileFormatError("Error parsing cluster. Offset of the first blob is too small.");
+    }
+
     size_t n_offset = offset / sizeof(OFFSET_TYPE);
 
+    if ( n_offset * sizeof(OFFSET_TYPE) != offset ) {
+        throw zim::ZimFileFormatError("Error parsing cluster. Offset of the first blob is not properly aligned.");
+    }
+
+    if ( maxBlobCount + 1 != 0 && n_offset > maxBlobCount + 1 ) {
+        throw zim::ZimFileFormatError("Error parsing cluster. Offset of the first blob is too large.");
+    }
+
     // read offsets
     m_blobOffsets.clear();
     m_blobOffsets.reserve(n_offset);
@@ -127,14 +137,25 @@ getClusterReader(const Reader& zimReader, offset_t offset, Cluster::Compression*
     }
   }
 
-  zsize_t Cluster::getBlobSize(blob_index_t n) const
+  void Cluster::checkBlobIndex(blob_index_t n) const
   {
       if (blob_index_type(n)+1 >= m_blobOffsets.size()) {
         throw ZimFileFormatError("blob index out of range");
       }
+  }
+
+  zsize_t Cluster::getBlobSize(blob_index_t n) const
+  {
+      checkBlobIndex(n);
       return zsize_t(m_blobOffsets[blob_index_type(n)+1].v - m_blobOffsets[blob_index_type(n)].v);
   }
 
+  offset_t Cluster::getBlobOffset(blob_index_t n) const
+  {
+      checkBlobIndex(n);
+      return offset_t(1) + m_blobOffsets[blob_index_type(n)];
+  }
+
   const Reader& Cluster::getReader(blob_index_t n) const
   {
     std::lock_guard<std::mutex> lock(m_readerAccessMutex);
@@ -179,4 +200,39 @@ getClusterReader(const Reader& zimReader, offset_t offset, Cluster::Compression*
     }
   }
 
+  // This function must return the memory consumption for a given cluster so
+  // that it can be used as a cost estimate during caching.
+  // However, because of partial (incremental) decompression, this size depends
+  // on the state of decompression:
+  // - As decompression advances, new blob readers are created in
+  //   `m_blobReaders`
+  // - The decoding/decompressing stream itself may allocate memory.
+  // Our approach is to return the average memory consumption by this cluster
+  // under the assumption that half of its data is decompressed.
+  // Note:
+  //  - No need to protect this method from concurent access (as well
+  //    as memoize its result) as it is intended to be called by ConcurentCache
+  //    which should invoke this method exactly once per cluster object
+  size_t Cluster::getMemorySize() const {
+    const auto blobOffsetsSize = sizeof(offset_t) * m_blobOffsets.size();
+    const auto decompressedDataSize = m_blobOffsets.back().v;
+
+    // If the cluster is not compressed, we rely on mmap and kernel
+    // to do the memory management.
+    const auto dataSize = isCompressed() ? decompressedDataSize : 0;
+
+    // Memory consumption by the decompressor stream.
+    // For non-compressed data reader it is assumed to be 0 (see the comment
+    // in BaseFileReader::getMemorySize())
+    auto streamSize = m_reader->getMemorySize();
+
+    // Compression levels may define a huge window and make decompression
+    // stream allocate a huge memory to store it. However (at least on
+    // linux) the actual memory used (as opposed to allocated memory) will
+    // not exceed the content size, even for a larger window. So, let's
+    // clamp the stream size to the size of the content itself.
+    streamSize = std::min<size_type>(streamSize, decompressedDataSize);
+
+    return blobOffsetsSize + dataSize/2 + streamSize;
+  }
 }
