@@ -20,6 +20,7 @@
 #define ZIM_PRIVATE
 #include <zim/zim.h>
 #include <zim/archive.h>
+#include <zim/illustration.h>
 #include <zim/item.h>
 #if defined(ENABLE_XAPIAN)
 #include <zim/search.h>
@@ -43,6 +44,18 @@ using zim::unittests::TempFile;
 using zim::unittests::TestItem;
 using zim::unittests::IsFrontArticle;
 
+class ZimArchive: public testing::Test {
+  protected:
+  void SetUp() override {
+    zim::setClusterCacheMaxSize(0);
+    zim::setClusterCacheMaxSize(CLUSTER_CACHE_SIZE);
+    ASSERT_EQ(zim::getClusterCacheCurrentSize(), 0);
+  }
+  void TearDown() override {
+    ASSERT_EQ(zim::getClusterCacheCurrentSize(), 0);
+  }
+};
+
 using TestContextImpl = std::vector<std::pair<std::string, std::string> >;
 struct TestContext : TestContextImpl {
   TestContext(const std::initializer_list<value_type>& il)
@@ -80,7 +93,7 @@ emptyZimArchiveContent()
   return content;
 }
 
-TEST(ZimArchive, openingAnInvalidZimArchiveFails)
+TEST_F(ZimArchive, openingAnInvalidZimArchiveFails)
 {
   const char* const prefixes[] = { "ZIM\x04", "" };
   const unsigned char bytes[] = {0x00, 0x01, 0x11, 0x30, 0xFF};
@@ -101,7 +114,7 @@ TEST(ZimArchive, openingAnInvalidZimArchiveFails)
   }
 }
 
-TEST(ZimArchive, openingAnEmptyZimArchiveSucceeds)
+TEST_F(ZimArchive, openingAnEmptyZimArchiveSucceeds)
 {
   const auto tmpfile = makeTempFile("empty_zim_file", emptyZimArchiveContent());
 
@@ -122,7 +135,7 @@ bool isNastyOffset(int offset) {
   return true;
 }
 
-TEST(ZimArchive, nastyEmptyZimArchive)
+TEST_F(ZimArchive, nastyEmptyZimArchive)
 {
   const std::string correctContent = emptyZimArchiveContent();
   for ( int offset = 0; offset < 80; ++offset ) {
@@ -136,7 +149,7 @@ TEST(ZimArchive, nastyEmptyZimArchive)
   }
 }
 
-TEST(ZimArchive, wrongChecksumInEmptyZimArchive)
+TEST_F(ZimArchive, wrongChecksumInEmptyZimArchive)
 {
   std::string zimfileContent = emptyZimArchiveContent();
   zimfileContent[85] = '\xff';
@@ -147,7 +160,7 @@ TEST(ZimArchive, wrongChecksumInEmptyZimArchive)
 }
 
 
-TEST(ZimArchive, openCreatedArchive)
+TEST_F(ZimArchive, openCreatedArchive)
 {
   TempFile temp("zimfile");
   auto tempPath = temp.path();
@@ -166,8 +179,8 @@ TEST(ZimArchive, openCreatedArchive)
   item = std::make_shared<TestItem>("foo2", "text/html", "AFoo", "Foo2Content", IsFrontArticle::NO);
   creator.addItem(item);
   creator.addMetadata("Title", "This is a title");
-  creator.addIllustration(48, "PNGBinaryContent48");
-  creator.addIllustration(96, "PNGBinaryContent96");
+  creator.addIllustration(zim::IllustrationInfo{48, 48, 1, {}}, "PNGBinaryContent48");
+  creator.addIllustration(zim::IllustrationInfo{96, 96, 1, {}}, "PNGBinaryContent96");
   creator.setMainPath("foo");
   creator.addRedirection("foo3", "FooRedirection", "foo"); // No a front article.
   creator.addRedirection("foo4", "FooRedirection", "NoExistant"); // Invalid redirection, must be removed by creator
@@ -175,11 +188,11 @@ TEST(ZimArchive, openCreatedArchive)
 
   zim::Archive archive(tempPath);
 #if !defined(ENABLE_XAPIAN)
-// 2*listingIndex + M/Counter + M/Title + mainpage + 2*Illustration + 2*Item + redirection
-#define ALL_ENTRY_COUNT 10U
+// listingIndex + M/Counter + M/Title + mainpage + 2*Illustration + 2*Item + redirection
+#define ALL_ENTRY_COUNT 9U
 #else
 // same as above + 2 xapian indexes.
-#define ALL_ENTRY_COUNT 12U
+#define ALL_ENTRY_COUNT 11U
 #endif
   ASSERT_EQ(archive.getAllEntryCount(), ALL_ENTRY_COUNT);
 #undef ALL_ENTRY_COUNT
@@ -187,7 +200,13 @@ TEST(ZimArchive, openCreatedArchive)
   ASSERT_EQ(archive.getArticleCount(), 1U);
   ASSERT_EQ(archive.getUuid(), uuid);
   ASSERT_EQ(archive.getMetadataKeys(), std::vector<std::string>({"Counter", "Illustration_48x48@1", "Illustration_96x96@1", "Title"}));
-  ASSERT_EQ(archive.getIllustrationSizes(), std::set<unsigned int>({48, 96}));
+  ASSERT_EQ(archive.getIllustrationInfos(), zim::Archive::IllustrationInfos({
+              {48, 48, 1.0, {}},
+              {96, 96, 1.0, {}},
+  }));
+  ASSERT_EQ(archive.getIllustrationInfos(48, 48, 1), zim::Archive::IllustrationInfos({
+              {48, 48, 1.0, {}},
+  }));
   ASSERT_TRUE(archive.hasMainEntry());
 
   ASSERT_EQ(archive.getMetadata("Title"), "This is a title");
@@ -200,13 +219,13 @@ TEST(ZimArchive, openCreatedArchive)
 
   ASSERT_EQ(archive.getMetadata("Counter"), "text/html=2");
 
-  auto illu48 = archive.getIllustrationItem(48);
+  auto illu48 = archive.getIllustrationItem(zim::IllustrationInfo{48, 48, 1.0, {}});
   ASSERT_EQ(illu48.getPath(), "Illustration_48x48@1");
   ASSERT_EQ(std::string(illu48.getData()), "PNGBinaryContent48");
   auto illu48Meta = archive.getMetadataItem(illu48.getPath());
   ASSERT_EQ(std::string(illu48Meta.getData()), "PNGBinaryContent48");
   ASSERT_EQ(illu48Meta.getMimetype(), "image/png");
-  auto illu96 = archive.getIllustrationItem(96);
+  auto illu96 = archive.getIllustrationItem(zim::IllustrationInfo{96, 96, 1.0, {}});
   ASSERT_EQ(illu96.getPath(), "Illustration_96x96@1");
   ASSERT_EQ(std::string(illu96.getData()), "PNGBinaryContent96");
 
@@ -245,13 +264,13 @@ TEST(ZimArchive, openCreatedArchive)
 }
 
 #if WITH_TEST_DATA
-TEST(ZimArchive, openRealZimArchive)
+TEST_F(ZimArchive, openRealZimArchive)
 {
   const char* const zimfiles[] = {
     "small.zim",
     "wikibooks_be_all_nopic_2017-02.zim",
     "wikibooks_be_all_nopic_2017-02_splitted.zim",
-    "wikipedia_en_climate_change_nopic_2020-01.zim"
+    "wikipedia_en_climate_change_mini_2024-06.zim"
   };
 
   for ( const std::string fname : zimfiles ) {
@@ -266,7 +285,7 @@ TEST(ZimArchive, openRealZimArchive)
   }
 }
 
-TEST(ZimArchive, openSplitZimArchive)
+TEST_F(ZimArchive, openSplitZimArchive)
 {
   const char* fname = "wikibooks_be_all_nopic_2017-02_splitted.zim";
 
@@ -280,7 +299,241 @@ TEST(ZimArchive, openSplitZimArchive)
   }
 }
 
-TEST(ZimArchive, openDontFallbackOnNonSplitZimArchive)
+struct TestCacheConfig {
+  size_t direntCacheSize;
+  size_t clusterCacheSize;
+  size_t direntLookupCacheSize;
+};
+
+struct RefEntry {
+  void test_is_equal(const zim::Entry& entry) {
+    ASSERT_EQ(path, entry.getPath());
+    ASSERT_EQ(title, entry.getTitle());
+    ASSERT_EQ(isRedirect, entry.isRedirect());
+    if (isRedirect) {
+      zim::entry_index_type redirectId = redirect_or_hash;
+      ASSERT_EQ(redirectId, entry.getRedirectEntryIndex());
+    } else {
+      auto hash = std::hash<std::string>{}(std::string(entry.getItem().getData()));
+      ASSERT_EQ(redirect_or_hash, hash);
+    }
+  }
+
+  std::string path;
+  std::string title;
+  bool        isRedirect;
+  // size_t is either 32 or 64 bits and entry_index_type (redirect id) is always 32 bits.
+  size_t      redirect_or_hash;
+};
+
+struct RefArchiveContent {
+  RefArchiveContent(const zim::Archive& archive) {
+    for (auto entry:archive.iterEfficient()) {
+      RefEntry ref_entry = {
+          entry.getPath(),
+          entry.getTitle(),
+          entry.isRedirect(),
+          entry.isRedirect() ? entry.getRedirectEntryIndex() : std::hash<std::string>{}(std::string(entry.getItem().getData())),
+      };
+      ref_entries.push_back(ref_entry);
+    }
+  }
+
+  void test_is_equal(const zim::Archive& archive) {
+    for (auto ref_entry:ref_entries) {
+      auto entry = archive.getEntryByPath(ref_entry.path);
+      ref_entry.test_is_equal(entry);
+    }
+  }
+  std::vector<RefEntry> ref_entries;
+};
+
+
+TEST_F(ZimArchive, cacheDontImpactReading)
+{
+  const TestCacheConfig cacheConfigs[] = {
+    {0, 0, 0},
+    {1, 1<<20, 1},
+    {2, 2<<20, 2},
+    {10, 10<<20, 10},
+    {1000, 2000<<20, 1000},
+    {0, 2000<<20, 1000},
+    {1000, 0<<20, 1000},
+    {1000, 2000<<20, 0},
+    {1, 2000<<20, 1000},
+    {1000, 1<<20, 1000},
+    {1000, 2000<<20, 1},
+  };
+
+  for (auto& testfile: getDataFilePath("small.zim")) {
+    RefArchiveContent ref_archive(zim::Archive(testfile.path));
+
+    for (auto cacheConfig: cacheConfigs) {
+      auto test_archive = zim::Archive(testfile.path, zim::OpenConfig().preloadDirentRanges(cacheConfig.direntLookupCacheSize));
+      test_archive.setDirentCacheMaxSize(cacheConfig.direntCacheSize);
+      zim::setClusterCacheMaxSize(cacheConfig.clusterCacheSize);
+
+      EXPECT_EQ(test_archive.getDirentCacheMaxSize(), cacheConfig.direntCacheSize);
+      EXPECT_EQ(zim::getClusterCacheMaxSize(), cacheConfig.clusterCacheSize);
+
+      ref_archive.test_is_equal(test_archive);
+    }
+  }
+}
+
+TEST_F(ZimArchive, cacheClean) {
+  for (auto& testfile: getDataFilePath("wikibooks_be_all_nopic_2017-02.zim")) {
+    EXPECT_EQ(zim::getClusterCacheCurrentSize(), 0); // No clusters in cache
+    {
+      auto archive = zim::Archive(testfile.path);
+      auto range = archive.iterEfficient();
+      auto it = range.begin();
+      for (auto i = 0; i<50 && it != range.end(); i++, it++) {
+        // Be sure to search by path to populate the dirent cache
+        auto entry = archive.getEntryByPath(it->getPath());
+        auto item = entry.getItem(true);
+        auto data = item.getData();
+      }
+      EXPECT_GT(zim::getClusterCacheCurrentSize(), 0); // No clusters in cache
+    }
+    EXPECT_EQ(zim::getClusterCacheCurrentSize(), 0); // No clusters in cache
+  }
+}
+
+TEST_F(ZimArchive, cacheChange)
+{
+  // We test only one variant here.
+  // Each variant has cluster of different size (especially the old "withns" which
+  // have a cluster compressed with a algorithm/compression level making input stream
+  // having a size of 64MB),
+  // this make all the following reasoning about cluster size a bit too complex.
+  // As the test here don't test that we can read all variant, we don't have too.
+  for (auto& testfile: getDataFilePath("wikibooks_be_all_nopic_2017-02.zim", {"noTitleListingV0"})) {
+    // wikibooks has only 2 clusters. One of 492121 bytes and one of 823716 bytes.
+    // For a total of 1315837 bytes.
+    // Has we try to keep one cluster in the cache, any size under the size of one
+    // cluster will not be respected.
+    // So we will define 2 limits:
+    // 850<<10 : size higher than a cluster size but under 2
+    // 2 << 20 : size higher than two clusters
+    const size_t L1_SIZE = 850 << 10;
+    const size_t L2_SIZE = 2 << 20;
+
+    EXPECT_EQ(zim::getClusterCacheCurrentSize(), 0);
+    RefArchiveContent ref_archive(zim::Archive(testfile.path));
+    auto archive = zim::Archive(testfile.path);
+
+    archive.setDirentCacheMaxSize(30);
+    zim::setClusterCacheMaxSize(L2_SIZE);
+
+    auto ref_it = ref_archive.ref_entries.begin();
+    for (auto i = 0; i<50 && ref_it != ref_archive.ref_entries.end(); i++, ref_it++) {
+      auto entry = archive.getEntryByPath(ref_it->path);
+      ref_it->test_is_equal(entry);
+    }
+    EXPECT_EQ(archive.getDirentCacheCurrentSize(), 30);
+    EXPECT_LE(zim::getClusterCacheCurrentSize(), L2_SIZE); // Only 2 clusters in the file
+
+    // Reduce cache size
+    archive.setDirentCacheMaxSize(10);
+    zim::setClusterCacheMaxSize(L1_SIZE);
+
+    EXPECT_EQ(archive.getDirentCacheCurrentSize(), 10);
+    EXPECT_LE(zim::getClusterCacheCurrentSize(), L1_SIZE);
+
+    // We want to test change of cache while we are iterating on the archive.
+    // So we don't reset the ref_it to `range.begin()`.
+    for (auto i = 0; i<50 && ref_it != ref_archive.ref_entries.end(); i++, ref_it++) {
+      auto entry = archive.getEntryByPath(ref_it->path);
+      ref_it->test_is_equal(entry);
+    }
+
+    EXPECT_EQ(archive.getDirentCacheCurrentSize(), 10);
+    EXPECT_LE(zim::getClusterCacheCurrentSize(), L1_SIZE);
+
+    // Clean cache
+    // (More than testing the value, this is needed as we want to be sure the cache is actually populated later)
+    archive.setDirentCacheMaxSize(0);
+    zim::setClusterCacheMaxSize(0);
+
+    EXPECT_EQ(archive.getDirentCacheCurrentSize(), 0);
+    EXPECT_EQ(zim::getClusterCacheCurrentSize(), 0);
+
+    // Increase the cache
+    archive.setDirentCacheMaxSize(20);
+    zim::setClusterCacheMaxSize(L1_SIZE);
+    EXPECT_EQ(archive.getDirentCacheCurrentSize(), 0);
+    EXPECT_EQ(zim::getClusterCacheCurrentSize(), 0);
+
+    ref_archive.test_is_equal(archive);
+    EXPECT_EQ(archive.getDirentCacheCurrentSize(), 20);
+    EXPECT_LE(zim::getClusterCacheCurrentSize(), L1_SIZE);
+  }
+}
+
+
+TEST_F(ZimArchive, MultiZimCache)
+{
+  // Get a list of several zim files to open (whatever the variant)
+  std::vector<std::string> zimPaths;
+  const char* const zimfiles[] = {
+    "wikibooks_be_all_nopic_2017-02.zim",
+    "wikibooks_be_all_nopic_2017-02_splitted.zim",
+    "wikipedia_en_climate_change_mini_2024-06.zim"
+  };
+
+  for ( const std::string fname : zimfiles ) {
+    for (auto& testfile: getDataFilePath(fname)) {
+      zimPaths.push_back(testfile.path);
+    }
+  }
+
+
+  const size_t SMALL_LIMIT = 5 << 20;
+  const size_t BIG_LIMIT = 200 << 20;
+  zim::setClusterCacheMaxSize(BIG_LIMIT);
+
+  std::vector<zim::Archive> archives;
+  for (auto path:zimPaths) {
+    auto archive = zim::Archive(path);
+    for (auto entry:archive.iterEfficient()) {
+      auto item = entry.getItem(true);
+      auto data = item.getData();
+    }
+    archives.push_back(archive);
+  }
+
+  EXPECT_LE(zim::getClusterCacheCurrentSize(), BIG_LIMIT);
+  zim::setClusterCacheMaxSize(SMALL_LIMIT);
+  EXPECT_LE(zim::getClusterCacheCurrentSize(), SMALL_LIMIT);
+
+  // Opening an archive should increase the cluster cache
+  zim::setClusterCacheMaxSize(BIG_LIMIT);
+  auto current_limit = zim::getClusterCacheCurrentSize();
+  {
+    auto archive = zim::Archive(zimPaths[0]);
+    for (auto entry:archive.iterEfficient()) {
+      auto item = entry.getItem(true);
+      auto data = item.getData();
+    }
+    EXPECT_GT(zim::getClusterCacheCurrentSize(), current_limit);
+    current_limit = zim::getClusterCacheCurrentSize();
+  }
+  // Destroying an archive should decrease the cluster cache
+  EXPECT_LT(zim::getClusterCacheCurrentSize(), current_limit);
+
+  // Be sure that decreasing the number of archives open also decrease the
+  // current cache size, until we reach 0.
+  current_limit = zim::getClusterCacheCurrentSize();
+  while (!archives.empty()) {
+    archives.pop_back();
+    EXPECT_LE(zim::getClusterCacheCurrentSize(), current_limit);
+    current_limit = zim::getClusterCacheCurrentSize();
+  }
+  EXPECT_EQ(zim::getClusterCacheCurrentSize(), 0);
+}
+
+TEST_F(ZimArchive, openDontFallbackOnNonSplitZimArchive)
 {
   const char* fname = "wikibooks_be_all_nopic_2017-02.zim";
 
@@ -296,7 +549,7 @@ TEST(ZimArchive, openDontFallbackOnNonSplitZimArchive)
   }
 }
 
-TEST(ZimArchive, openNonExistantZimArchive)
+TEST_F(ZimArchive, openNonExistantZimArchive)
 {
   const std::string fname = "non_existant.zim";
 
@@ -309,7 +562,7 @@ TEST(ZimArchive, openNonExistantZimArchive)
   }
 }
 
-TEST(ZimArchive, openNonExistantZimSplitArchive)
+TEST_F(ZimArchive, openNonExistantZimSplitArchive)
 {
   const std::string fname = "non_existant.zimaa";
 
@@ -322,12 +575,12 @@ TEST(ZimArchive, openNonExistantZimSplitArchive)
   }
 }
 
-TEST(ZimArchive, randomEntry)
+TEST_F(ZimArchive, randomEntry)
 {
   const char* const zimfiles[] = {
     "wikibooks_be_all_nopic_2017-02.zim",
     "wikibooks_be_all_nopic_2017-02_splitted.zim",
-    "wikipedia_en_climate_change_nopic_2020-01.zim"
+    "wikipedia_en_climate_change_mini_2024-06.zim"
   };
 
   for ( const std::string fname : zimfiles ) {
@@ -347,7 +600,7 @@ TEST(ZimArchive, randomEntry)
   }
 }
 
-TEST(ZimArchive, illustration)
+TEST_F(ZimArchive, illustration)
 {
   const char* const zimfiles[] = {
     "small.zim",
@@ -360,12 +613,14 @@ TEST(ZimArchive, illustration)
       const zim::Archive archive(testfile.path);
       ASSERT_TRUE(archive.hasIllustration(48)) << ctx;
       auto illustrationItem = archive.getIllustrationItem(48);
-      if(testfile.category == "nons") {
-        ASSERT_EQ(illustrationItem.getPath(), "Illustration_48x48@1") << ctx;
-      } else {
+      if(testfile.category == "withns") {
         ASSERT_EQ(illustrationItem.getPath(), "I/favicon.png") << ctx;
+      } else {
+        ASSERT_EQ(illustrationItem.getPath(), "Illustration_48x48@1") << ctx;
       }
-      ASSERT_EQ(archive.getIllustrationSizes(), std::set<unsigned int>({48}));
+      ASSERT_EQ(archive.getIllustrationInfos(), zim::Archive::IllustrationInfos({
+                  {48, 48, 1.0, {}},
+      }));
     }
   }
 }
@@ -377,7 +632,7 @@ struct ZimFileInfo {
 struct TestDataInfo {
   const char* const name;
   zim::entry_index_type mediaCount;
-  ZimFileInfo withnsInfo, nonsInfo;
+  ZimFileInfo withnsInfo, nonsInfo, noTitleListingV0Info;
 
 
   const ZimFileInfo& getZimFileInfo(const std::string& category) const {
@@ -385,20 +640,25 @@ struct TestDataInfo {
       return nonsInfo;
     } else if (category == "withns") {
       return withnsInfo;
+    } else if (category == "noTitleListingV0") {
+      return noTitleListingV0Info;
     }
     throw std::runtime_error("Unknown category");
   }
 };
 
-TEST(ZimArchive, articleNumber)
+TEST_F(ZimArchive, articleNumber)
 {
   TestDataInfo zimfiles[] = {
-     // Name                                          mediaCount,  withns                                           nons
-     //                                                            {articles, userEntries, allEntries}, {articles, userEntries, allEntries}
-    {"small.zim",                                     1,           { 1,       17,          17,       }, { 1,       2,           16        }},
-    {"wikibooks_be_all_nopic_2017-02.zim",            34,          { 66,      118,         118,      }, { 66,      109,         123       }},
-    {"wikibooks_be_all_nopic_2017-02_splitted.zim",   34,          { 66,      118,         118,      }, { 66,      109,         123       }},
-    {"wikipedia_en_climate_change_nopic_2020-01.zim", 333,         { 1837,    7646,        7646,     }, { 1837,    7633,        7649      }}
+     // Name                                          mediaCount,  withns                               nons                                 noTitleListingV0
+     //                                                            {articles, userEntries, allEntries}, {articles, userEntries, allEntries}, {articles, userEntries, allEntries}
+    {"small.zim",                                     1,           { 1,       17,          17,       }, { 1,       2,           16        }, { 1,       2,           16        }},
+// For some unknown reason, nons wikibooks already doesn't contain a v0 title index so number of allEntries is equal to noTitleListingV0.
+// But header titlePtrPos is initialized in nons and is 0 in noTitleListingV0.
+// I suspect here that nons file was generated using a local dev buggy zimrecreate.
+    {"wikibooks_be_all_nopic_2017-02.zim",            34,          { 66,      118,         118,      }, { 66,      109,         123       }, { 66,      109,         123       }},
+    {"wikibooks_be_all_nopic_2017-02_splitted.zim",   34,          { 66,      118,         118,      }, { 66,      109,         123       }, { 66,      109,         123       }},
+    {"wikipedia_en_climate_change_mini_2024-06.zim",  111,         { 3821,    20565,       20565,    }, { 3821,    20551,       20568     }, { 3821,    20551,       20567     }}
   };
   // "withns" zim files have no notion of user entries, so EntryCount == allEntryCount.
   // for small.zim, there is always 1 article, whatever the article is in 'A' namespace or in specific index.
@@ -441,13 +701,20 @@ public:
 #define EXPECT_BROKEN_ZIMFILE(ZIMPATH, EXPECTED_STDERROR_TEXT) \
   CapturedStderr stderror;                                     \
   EXPECT_FALSE(zim::validate(ZIMPATH, checksToRun));           \
-  EXPECT_EQ(EXPECTED_STDERROR_TEXT, std::string(stderror)) << ZIMPATH;
+  EXPECT_EQ(EXPECTED_STDERROR_TEXT, std::string(stderror)) << ZIMPATH; \
+  ASSERT_EQ(zim::getClusterCacheCurrentSize(), 0);
 
 #define TEST_BROKEN_ZIM_NAME(ZIMNAME, EXPECTED)                \
 for(auto& testfile: getDataFilePath(ZIMNAME)) {EXPECT_BROKEN_ZIMFILE(testfile.path, EXPECTED)}
 
+#define TEST_BROKEN_ZIM_NAME_CAT(ZIMNAME, CAT, EXPECTED)                \
+for(auto& testfile: getDataFilePath(ZIMNAME, CAT)) {EXPECT_BROKEN_ZIMFILE(testfile.path, EXPECTED)}
+
+// Use an intermediate define to avoid CPP to interpreted the comma as a parameter separator.
+#define WITH_TITLE_IDX_CAT {"withns", "nons"}
+
 #if WITH_TEST_DATA
-TEST(ZimArchive, validate)
+TEST_F(ZimArchive, validate)
 {
   zim::IntegrityCheckList all;
   all.set();
@@ -474,8 +741,10 @@ TEST(ZimArchive, validate)
     std::string expected;
     if (testfile.category == "withns") {
       expected = "Title index table outside (or not fully inside) ZIM file.\n";
-    } else {
+    } else if ( testfile.category == "nons") {
       expected = "Full Title index table outside (or not fully inside) ZIM file.\n";
+    } else {
+      continue;
     }
     EXPECT_BROKEN_ZIMFILE(testfile.path, expected)
   }
@@ -505,13 +774,15 @@ TEST(ZimArchive, validate)
     "Invalid dirent pointer\n"
   );
 
-  TEST_BROKEN_ZIM_NAME(
+  TEST_BROKEN_ZIM_NAME_CAT(
     "invalid.outofbounds_first_title_entry.zim",
+    WITH_TITLE_IDX_CAT,
     "Invalid title index entry.\n"
   );
 
-  TEST_BROKEN_ZIM_NAME(
+  TEST_BROKEN_ZIM_NAME_CAT(
     "invalid.outofbounds_last_title_entry.zim",
+    WITH_TITLE_IDX_CAT,
     "Invalid title index entry.\n"
   );
 
@@ -520,6 +791,43 @@ TEST(ZimArchive, validate)
     "Invalid cluster pointer\n"
   );
 
+  TEST_BROKEN_ZIM_NAME(
+    "invalid.too_small_offset_of_first_blob_in_cluster_0.zim",
+     "Error parsing cluster. Offset of the first blob is too small.\n"
+  )
+
+#if defined(ENABLE_XAPIAN)
+  TEST_BROKEN_ZIM_NAME(
+    "invalid.too_small_offset_of_first_blob_in_cluster_4.zim",
+     "blob index out of range\n"
+  )
+#endif
+
+  TEST_BROKEN_ZIM_NAME(
+    "invalid.too_small_offset_of_first_blob_in_cluster_7.zim",
+     "Error parsing cluster. Offset of the first blob is too small.\n"
+  )
+
+  TEST_BROKEN_ZIM_NAME(
+    "invalid.misaligned_offset_of_first_blob_in_cluster_9.zim",
+     "Error parsing cluster. Offset of the first blob is not properly aligned.\n"
+  )
+
+  TEST_BROKEN_ZIM_NAME(
+    "invalid.misaligned_offset_of_first_blob_in_cluster_10.zim",
+     "Error parsing cluster. Offset of the first blob is not properly aligned.\n"
+  )
+
+  TEST_BROKEN_ZIM_NAME(
+    "invalid.misaligned_offset_of_first_blob_in_cluster_11.zim",
+     "Error parsing cluster. Offset of the first blob is not properly aligned.\n"
+  )
+
+  TEST_BROKEN_ZIM_NAME(
+    "invalid.too_large_offset_of_first_blob_in_cluster.zim",
+     "Error parsing cluster. Offset of the first blob is too large.\n"
+  )
+
   TEST_BROKEN_ZIM_NAME(
     "invalid.offset_in_cluster.zim",
      "Error parsing cluster. Offsets are not ordered.\n"
@@ -540,8 +848,9 @@ TEST(ZimArchive, validate)
     EXPECT_BROKEN_ZIMFILE(testfile.path, expected)
   }
 
-  TEST_BROKEN_ZIM_NAME(
+  TEST_BROKEN_ZIM_NAME_CAT(
     "invalid.nonsorted_title_index.zim",
+    WITH_TITLE_IDX_CAT,
     "Title index is not properly sorted.\n"
   );
 
@@ -554,8 +863,10 @@ TEST(ZimArchive, validate)
     std::string expected;
     if (testfile.category == "withns") {
       expected = "Entry M/Language has invalid MIME-type value 1234.\n";
-    } else {
+    } else if (testfile.category == "nons") {
       expected = "Entry M/Publisher has invalid MIME-type value 1234.\n";
+    } else {
+      expected = "Entry M/Name has invalid MIME-type value 1234.\n";
     }
     EXPECT_BROKEN_ZIMFILE(testfile.path, expected)
   }
@@ -638,7 +949,7 @@ void checkEquivalence(const zim::Archive& archive1, const zim::Archive& archive2
 #endif
 }
 
-TEST(ZimArchive, multipart)
+TEST_F(ZimArchive, multipart)
 {
   auto nonSplittedZims = getDataFilePath("wikibooks_be_all_nopic_2017-02.zim");
   auto splittedZims = getDataFilePath("wikibooks_be_all_nopic_2017-02_splitted.zim");
@@ -667,7 +978,7 @@ TEST(ZimArchive, multipart)
 #endif
 
 #ifndef _WIN32
-TEST(ZimArchive, openByFD)
+TEST_F(ZimArchive, openByFD)
 {
   for(auto& testfile: getDataFilePath("small.zim")) {
     const zim::Archive archive1(testfile.path);
@@ -678,7 +989,7 @@ TEST(ZimArchive, openByFD)
   }
 }
 
-TEST(ZimArchive, openZIMFileEmbeddedInAnotherFile)
+TEST_F(ZimArchive, openZIMFileEmbeddedInAnotherFile)
 {
   auto normalZims = getDataFilePath("small.zim");
   auto embeddedZims = getDataFilePath("small.zim.embedded");
@@ -694,7 +1005,7 @@ TEST(ZimArchive, openZIMFileEmbeddedInAnotherFile)
   }
 }
 
-TEST(ZimArchive, openZIMFileMultiPartEmbeddedInAnotherFile)
+TEST_F(ZimArchive, openZIMFileMultiPartEmbeddedInAnotherFile)
 {
   auto normalZims = getDataFilePath("small.zim");
   auto embeddedZims = getDataFilePath("small.zim.embedded.multi");
@@ -729,15 +1040,15 @@ TEST(ZimArchive, openZIMFileMultiPartEmbeddedInAnotherFile)
 
 
 #if WITH_TEST_DATA
-zim::Blob readItemData(const zim::Item::DirectAccessInfo& dai, zim::size_type size)
+zim::Blob readItemData(const zim::ItemDataDirectAccessInfo& dai, zim::size_type size)
 {
-  zim::DEFAULTFS::FD fd(zim::DEFAULTFS::openFile(dai.first));
+  zim::DEFAULTFS::FD fd(zim::DEFAULTFS::openFile(dai.filename));
   std::shared_ptr<char> data(new char[size], std::default_delete<char[]>());
-  fd.readAt(data.get(), zim::zsize_t(size), zim::offset_t(dai.second));
+  fd.readAt(data.get(), zim::zsize_t(size), zim::offset_t(dai.offset));
   return zim::Blob(data, size);
 }
 
-TEST(ZimArchive, getDirectAccessInformation)
+TEST_F(ZimArchive, getDirectAccessInformation)
 {
   for(auto& testfile:getDataFilePath("small.zim")) {
     const zim::Archive archive(testfile.path);
@@ -747,7 +1058,7 @@ TEST(ZimArchive, getDirectAccessInformation)
         const TestContext ctx{ {"entry", entry.getPath() } };
         const auto item = entry.getItem();
         const auto dai = item.getDirectAccessInformation();
-        if ( dai.first != "" ) {
+        if ( dai.isValid() ) {
           ++checkedItemCount;
           EXPECT_EQ(item.getData(), readItemData(dai, item.getSize())) << ctx;
         }
@@ -758,7 +1069,7 @@ TEST(ZimArchive, getDirectAccessInformation)
 }
 
 #ifndef _WIN32
-TEST(ZimArchive, getDirectAccessInformationInAnArchiveOpenedByFD)
+TEST_F(ZimArchive, getDirectAccessInformationInAnArchiveOpenedByFD)
 {
   for(auto& testfile:getDataFilePath("small.zim")) {
     const int fd = OPEN_READ_ONLY(testfile.path);
@@ -769,7 +1080,7 @@ TEST(ZimArchive, getDirectAccessInformationInAnArchiveOpenedByFD)
         const TestContext ctx{ {"entry", entry.getPath() } };
         const auto item = entry.getItem();
         const auto dai = item.getDirectAccessInformation();
-        if ( dai.first != "" ) {
+        if ( dai.isValid() ) {
           ++checkedItemCount;
           EXPECT_EQ(item.getData(), readItemData(dai, item.getSize())) << ctx;
         }
@@ -779,7 +1090,7 @@ TEST(ZimArchive, getDirectAccessInformationInAnArchiveOpenedByFD)
   }
 }
 
-TEST(ZimArchive, getDirectAccessInformationFromEmbeddedArchive)
+TEST_F(ZimArchive, getDirectAccessInformationFromEmbeddedArchive)
 {
   auto normalZims = getDataFilePath("small.zim");
   auto embeddedZims = getDataFilePath("small.zim.embedded");
@@ -795,7 +1106,7 @@ TEST(ZimArchive, getDirectAccessInformationFromEmbeddedArchive)
         const TestContext ctx{ {"entry", entry.getPath() } };
         const auto item = entry.getItem();
         const auto dai = item.getDirectAccessInformation();
-        if ( dai.first != "" ) {
+        if ( dai.isValid() ) {
           ++checkedItemCount;
           EXPECT_EQ(item.getData(), readItemData(dai, item.getSize())) << ctx;
         }
